/*
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

//*****************************************************************************
//  Filename:	 dlmsfast_t.c
//  Version:	 0.01
//  Description: test for fast dlms routine
//-----------------------------------------------------------------------------
//  Revision History:	 
//    1.000, Li Yuan, 09/23/02, - original release 
//    2 .00 R. Katzur   03/30/16   Benchmark
//*****************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <tms320.h>
#include <stdio.h>
#include <dsplib.h>
#include <time.h>

#define  NUMBER_OF_ITERATIONS   1000l


//#include "t3.h"   // 12,20
#include "t4.h"   // 32,64
//#include "t6.h"   // 12,100
//#include "t7.h"	// 10,50


//#include "t8.h"
//#include "test.h"  // 32,64

short test(DATA *r, DATA *rtest, short n, DATA maxerror);

short eflagr = PASS;
short eflagh = PASS;

void clear_ran(short x[], int N)
{
	int i   ;
	for (i=0; i<N; i++)
	{
		x[i] = 0   ;
	}
	return ;
}

void main()
{
    short i;
	   int nx ,nh ,nstep;
	   long  iterations1   ;
	   clock_t   t1,t2, t11,t22 ,total1_t ,total2_t,diff  ;

    // clear
//    for (i = 0; i< NH; i++) h[i] = 0;	// clear coeff buffer (optional)
//    for (i = 0; i< NX; i++) r[i] = 0;	// clear output buffer (optional)
    nx = NX  ;
    nh = NH   ;
    nstep = STEP   ;
    clear_ran(h, NH) ;
    clear_ran(r, NX) ;
    

    dbuffer[0] = 0;             // clear index
 //   for (i = 1; i < NH+3; i++) dbuffer[i] = 0; // clear delay buffer (a must)


    clear_ran(dbuffer, NH + 3) ;

    t1 = clock()   ;
    t2 = clock()  ;
    diff = t2 - t1 ;   ///   overhead of calling
    t1 = clock()   ;

    // compute
    dlmsfast(x, h, r, des, dbuffer, nstep, nh, nx);

    t2 = clock()  ;

     total1_t = (double) (t2 - t1-diff)   ;
     printf("fast LMS  Delay filter #values %d  #of taps %d  step %d \n" , nx, nh,nstep ) ;
     printf("  time (in cycles)  %ld \n", total1_t) ;
    
    // test
    eflagr = test(r, rtest, NX, MAXERROR);	// for r
    eflagh = test(h, htest, NH, MAXERROR);	// for h

	if(eflagr != PASS || eflagh != PASS){
        printf("FAIL THE TEST   \n");
	}
    for (iterations1 = 0; iterations1 < NUMBER_OF_ITERATIONS;iterations1++)

    {

        dlmsfast(x, h, r, des, dbuffer, nstep, nh, nx);

    }


    printf("Done with %ld iteration  \n",iterations1 );

    return;
}
